# stage_revert.py
#
# Copyright (C) 2014 Tim Marston <tim@edm.am>
#
# This file is part of stdhome (hereafter referred to as "this program").
# See http://ed.am/dev/stdhome for more information.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


import sys, re, getopt
from command import Command
import stdhome.the as the
from stdhome.deployment import Deployment


class StageRevertCommand( Command ):


	def __init__( self ):
		self.force = False


	def print_help( self ):
		print "Usage: " + the.program.name + " stage-revert [--repo=REPO]"
		print
		#      01234567890123456789012345678901234567890123456789012345678901234567890123456789
		print "Revert any modifications made to a local repository, losing those changes."
		print
		print "In addition to using the add and remove commands, you can also manually modify"
		print "the files in a local repository and then commit them to a remote repository as"
		print "required.  This can be done with the staging commands."
		print
		print "When files in your local repository have been modified, several of the primary"
		print "commands of this program will not be able to function.  This command reverts all"
		print "modifications to a local repository so that the primary commands can work again."
		print "If you have not used any of the staging commands, it is safe to revert a local"
		print "repository."
		print
		print "Options:"
		print "      --force      force revert of stage (when there is a problem)"
		print "  -r, --repo=REPO  select the repo to check-out or create (defaults to 'home')"
		print "  -v, --verbose    display information about what is being done"
		print "      --help       display help and exit"
		exit( 0 )


	def parse_command_line( self ):
		opts, args = getopt.gnu_getopt(
			sys.argv[ 1: ], "r:v",
			[ "force", "repo=", "verbose", "help" ] )
		for opt, optarg in opts:
			if opt == "--force":
				self.force = True
			elif opt in [ '--repo', '-r' ]:
				if not re.match( '^[-a-zA-z0-9.]+$', optarg ):
					raise the.program.FatalError(
						'invalid repository name: ' + optarg )
				the.repo = optarg
			elif opt == "--help":
				self.print_help()

		# discard first argument (the command)
		args.pop( 0 )

		# remaining arguments
		if len( args ):
			raise the.program.UsageError( 'too many arguments' )


	def run( self ):

		# set up repo and check it exists
		the.repo.check_dir_exists()

		# initialise deployment
		deployment = Deployment()
		if deployment.is_ongoing() and deployment.get_initial_revno() is None \
		   and not self.force:
			raise RuntimeError(
				"can't revert ongoing deployment: initial revno is missing!\n"
				"use --force to override, but an update may miss changes!" )

		# check for local changes
		if the.verbose >= 1: print "reverting %s" % the.repo.dir
		the.repo.vcs.revert( deployment.get_initial_revno() )

		# remove deployment state
		deployment.remove_deployment_state()
